@@ -1,6 +1,6 @@
 ##
 ## R package splines2 by Wenjie Wang and Jun Yan
-## Copyright (C) 2016-2021
+## Copyright (C) 2016-2022
 ##
 ## This file is part of the R package splines2.
 ##
@@ -20,7 +20,7 @@
 ##' Generates the I-spline (integral of M-spline) basis matrix for a polynomial
 ##' spline or the corresponding derivatives of given order.
 ##'
-##' It is an implementation of the close form I-spline basis based on the
+##' It is an implementation of the closed-form I-spline basis based on the
 ##' recursion formula given by Ramsay (1988).
 ##'
 ##' @inheritParams bSpline
@@ -33,7 +33,7 @@
 ##'     functions are returned.  Notice that when using I-Spline for monotonic
 ##'     regression, \code{intercept = TRUE} should be set even when an intercept
 ##'     term is considered additional to the spline basis functions.
-##' @param derivs A non-negative integer specifying the order of derivatives of
+##' @param derivs A nonnegative integer specifying the order of derivatives of
 ##'     I-splines.
 ##'
 ##' @inherit bSpline return
@@ -55,7 +55,7 @@ iSpline <- function(x, df = NULL, knots = NULL, degree = 3L,
 {
     ## check inputs
     if ((derivs <- as.integer(derivs)) < 0) {
-        stop("The 'derivs' must be a non-negative integer.")
+        stop("The 'derivs' must be a nonnegative integer.")
     }
     if (derivs > 0) {
         return(mSpline(x = x,
@@ -125,6 +125,6 @@ iSpline <- function(x, df = NULL, knots = NULL, degree = 3L,
         row.names(out) <- name_x
     }
     ## add class
-    class(out) <- c("matrix", "iSpline")
+    class(out) <- c("matrix", "iSpline", "splines2")
     out
 }
