@@ -1,6 +1,6 @@
 ##
 ## R package splines2 by Wenjie Wang and Jun Yan
-## Copyright (C) 2016-2020
+## Copyright (C) 2016-2022
 ##
 ## This file is part of the R package splines2.
 ##
@@ -33,15 +33,35 @@ null2num0 <- function(x) {
 }
 
 ## check key attributions
-check_attr <- function(x, check_derivs = FALSE,
-                       attrs = c("x", "degree", "knots",
-                                 "Boundary.knots", "intercept"))
+check_attr <- function(x, attrs = c("x", "degree", "knots",
+                                    "Boundary.knots", "intercept"))
 {
-    if (check_derivs) {
-        attrs <- c(attrs, "derivs")
-    }
-    if (any(! attrs %in% names(attributes(x)))) {
-        stop("Key attributions are missing.", call. = FALSE)
+    idx <- ! attrs %in% names(attributes(x))
+    if (any(idx)) {
+        stop(wrapMessages(
+            sprintf("Missing attributes: %s.",
+                    paste(attrs[idx], collapse = ", "))
+        ), call. = FALSE)
     }
     invisible()
 }
+
+## return most of the attributes except x and class
+pred_attr <- function(x, except = c("x", "class", "dimnames")) {
+    out <- attributes(x)
+    out[! names(out) %in% except]
+}
+
+## simplified version of utils::modifyList with keep.null = TRUE
+modify_list <- function (x, val) {
+    stopifnot(is.list(x), is.list(val))
+    xnames <- names(x)
+    vnames <- names(val)
+    vnames <- vnames[nzchar(vnames)]
+    for (v in vnames) {
+        x[v] <- if (v %in% xnames && is.list(x[[v]]) && is.list(val[[v]]))
+                    list(modify_list(x[[v]], val[[v]]))
+                else val[v]
+    }
+    x
+}
