@@ -1,6 +1,6 @@
 //
 // R package splines2 by Wenjie Wang and Jun Yan
-// Copyright (C) 2016-2021
+// Copyright (C) 2016-2022
 //
 // This file is part of the R package splines2.
 //
@@ -29,29 +29,9 @@ namespace splines2 {
     // define a class for M-splines
     class MSpline : public SplineBase
     {
-        // inherits constructors
-        using SplineBase::SplineBase;
-
-    public:
-        // function members
-
-        //! Compute M-spline basis
-        //!
-        //! @param complete_basis A `bool` value indicating whether to return a
-        //! complete spline basis
-        //!
-        //! @return arma::mat
-        inline virtual rmat basis(const bool complete_basis = true)
+    protected:
+        inline rmat get_basis_simple()
         {
-            // early exit if latest
-            if (is_basis_latest_) {
-                if (complete_basis) {
-                    return spline_basis_;
-                }
-                // else
-                return mat_wo_col1(spline_basis_);
-            }
-            // else do the generation
             update_spline_df();
             update_x_index();
             update_knot_sequence();
@@ -63,7 +43,7 @@ namespace splines2 {
             for (size_t i {0}; i < x_.n_elem; ++i) {
                 unsigned int j { x_index_(i) };
                 double denom { knot_sequence_(j + order_) -
-                               knot_sequence_(j + degree_) };
+                    knot_sequence_(j + degree_) };
                 b_mat(i, j) = 1 / denom;
             }
             // main loop
@@ -81,59 +61,53 @@ namespace splines2 {
                         size_t j_index { x_index_(i) + j };
                         size_t i1 { j_index + k_offset };
                         size_t i2 { j_index + order_ };
-                        double den { knot_sequence_(i2) - knot_sequence_(i1) };
+                        double den {
+                            knot_sequence_(i2) - knot_sequence_(i1)
+                        };
                         double term { dk1 * b_mat(i, j_index) };
                         b_mat(i, j_index) = saved +
                             (knot_sequence_(i2) - x_(i)) * term / den;
                         double den2 {
-                            knot_sequence_(i2 + 1) - knot_sequence_(i1 + 1)
+                            knot_sequence_(i2 + 1) -
+                            knot_sequence_(i1 + 1)
                         };
-                        saved = (x_(i) - knot_sequence_(i1 + 1)) * term / den2;
+                        saved = (x_(i) - knot_sequence_(i1 + 1)) *
+                            term / den2;
                     }
                     b_mat(i, x_index_(i) + k) = saved;
                 }
             }
-            // about to return
-            spline_basis_ = b_mat;
-            is_basis_latest_ = true;
-            if (complete_basis) {
-                return b_mat;
-            }
-            // else
-            return mat_wo_col1(b_mat);
+            return b_mat;
         }
 
-        // derivatives of M-splines
-        inline virtual rmat derivative(
-            const unsigned int derivs = 1,
-            const bool complete_basis = true
+        inline rmat get_basis_extended()
+        {
+            MSpline msp_obj {
+                x_, surrogate_internal_knots_, degree_,
+                surrogate_boundary_knots_
+            };
+            rmat out { msp_obj.get_basis_simple() };
+            // remove first and last #degree basis functions
+            return out.cols(degree_, out.n_cols - order_);
+        }
+
+        inline rmat get_derivative_simple(
+            const unsigned int derivs = 1
             )
         {
-            if (derivs == 0) {
-                throw std::range_error(
-                    "'derivs' has to be a positive integer.");
-            }
-            // early exit if derivs is large enough
-            update_spline_df();
-            if (degree_ < derivs) {
-                if (complete_basis) {
-                    return arma::zeros(x_.n_elem, spline_df_);
-                }
-                if (spline_df_ == 1) {
-                    throw std::range_error("No column left in the matrix.");
-                }
-                return arma::zeros(x_.n_elem, spline_df_ - 1);
-            }
-            // create a copy of this object
-            MSpline ms_obj2 { this };
-            ms_obj2.set_degree(degree_ - derivs);
+            MSpline msp_obj { this };
+            msp_obj.set_degree(degree_ - derivs);
             // get basis matrix for (degree - derivs)
-            rmat d_mat { ms_obj2.basis(true) };
-            // add zero columns
-            d_mat = add_zero_cols(d_mat, spline_df_ - d_mat.n_cols);
+            rmat d_mat { msp_obj.get_basis_simple() };
             // make sure knot sequence and x index are latest
             update_knot_sequence();
             update_x_index();
+            // add zero columns
+            update_spline_df();
+            // if (spline_df_ <= d_mat.n_cols) {
+            //     throw std::range_error("FIXME: get_derivative_simple()");
+            // }
+            d_mat = add_zero_cols(d_mat, spline_df_ - d_mat.n_cols);
             // main loop
             for (unsigned int k {1}; k <= derivs; ++k) {
                 const unsigned int k_offset { derivs - k };
@@ -144,7 +118,9 @@ namespace splines2 {
                         size_t j_index { x_index_(i) + j };
                         size_t i1 { j_index + k_offset };
                         size_t i2 { j_index + order_ };
-                        double den { knot_sequence_(i2) - knot_sequence_(i1) };
+                        double den {
+                            knot_sequence_(i2) - knot_sequence_(i1)
+                        };
                         double term { (numer + 1) * d_mat(i, j_index) };
                         d_mat(i, j_index) = saved - term / den;
                         double den2 {
@@ -155,23 +131,38 @@ namespace splines2 {
                     d_mat(i, x_index_(i) + numer) = saved;
                 }
             }
-            // remove the first column if needed
-            if (complete_basis) {
-                return d_mat;
-            }
-            // else
-            return mat_wo_col1(d_mat);
+            return d_mat;
         }
 
-        // integral of M-splines (I-splines)
-        inline virtual rmat integral(const bool complete_basis = true)
+        inline rmat get_derivative_extended(
+            const unsigned int derivs = 1
+            )
+        {
+            MSpline msp_obj {
+                x_, surrogate_internal_knots_, degree_,
+                surrogate_boundary_knots_
+            };
+            rmat out { msp_obj.get_derivative_simple(derivs) };
+            // remove first and last #degree basis functions
+            return out.cols(degree_, out.n_cols - order_);
+        }
+
+        inline rmat get_integral_simple()
         {
             // create a copy of this object
-            MSpline ms_obj2 { this };
+            MSpline msp_obj { this };
             // get basis matrix for (degree - derivs)
-            ms_obj2.set_degree(degree_ + 1);
-            rmat i_mat { ms_obj2.basis(false) };
-            rvec knot_sequence_ord { ms_obj2.get_knot_sequence() };
+            msp_obj.set_degree(degree_ + 1);
+            rmat i_mat { msp_obj.basis(false) };
+            rvec knot_sequence_ord { msp_obj.get_knot_sequence() };
+            // throw warning if any x is less than left-most boundary
+            // if (arma::any(x_ < msp_obj.knot_sequence_(0))) {
+            //     Rcpp::Rcout << "Warning: Found x < the leftmost knot, "
+            //                 << msp_obj.knot_sequence_(0)
+            //                 << ". "
+            //                 << "The basis integrals were not well-defined."
+            //                 << std::endl;
+            // }
             // make sure x index is latest
             update_x_index();
             // compute t_{(i+1)+(k+1)+1} - t_{i+1} of s_{k}
@@ -197,6 +188,92 @@ namespace splines2 {
                     }
                 }
             }
+            return i_mat;
+        }
+
+        inline rmat get_integral_extended()
+        {
+            MSpline msp_obj {
+                x_, surrogate_internal_knots_, degree_,
+                surrogate_boundary_knots_
+            };
+            rmat out { msp_obj.get_integral_simple() };
+            // remove first and last #degree basis functions
+            return out.cols(degree_, out.n_cols - order_);
+        }
+
+    public:
+        // inherits constructors
+        using SplineBase::SplineBase;
+
+        // function members
+
+        //! Compute M-spline basis
+        //!
+        //! @param complete_basis A `bool` value indicating whether to return a
+        //! complete spline basis
+        //!
+        //! @return arma::mat
+        inline rmat basis(const bool complete_basis = true) override
+        {
+            rmat b_mat;
+            if (is_extended_knot_sequence_) {
+                b_mat = get_basis_extended();
+            } else {
+                b_mat = get_basis_simple();
+            }
+            // about to return
+            if (complete_basis) {
+                return b_mat;
+            }
+            // else
+            return mat_wo_col1(b_mat);
+        }
+
+        // derivatives of M-splines
+        inline rmat derivative(
+            const unsigned int derivs = 1,
+            const bool complete_basis = true
+            ) override
+        {
+            if (derivs == 0) {
+                throw std::range_error(
+                    "'derivs' has to be a positive integer.");
+            }
+            // early exit if derivs is large enough
+            update_spline_df();
+            if (degree_ < derivs) {
+                if (complete_basis) {
+                    return arma::zeros(x_.n_elem, spline_df_);
+                }
+                if (spline_df_ == 1) {
+                    throw std::range_error("No column left in the matrix.");
+                }
+                return arma::zeros(x_.n_elem, spline_df_ - 1);
+            }
+            rmat d_mat;
+            if (is_extended_knot_sequence_) {
+                d_mat = get_derivative_extended(derivs);
+            } else {
+                d_mat = get_derivative_simple(derivs);
+            }
+            // remove the first column if needed
+            if (complete_basis) {
+                return d_mat;
+            }
+            // else
+            return mat_wo_col1(d_mat);
+        }
+
+        // integral of M-splines (I-splines)
+        inline rmat integral(const bool complete_basis = true) override
+        {
+            rmat i_mat;
+            if (is_extended_knot_sequence_) {
+                i_mat = get_integral_extended();
+            } else {
+                i_mat = get_integral_simple();
+            }
             // remove the first column if needed
             if (complete_basis) {
                 return i_mat;
